import logging
from logging import LogRecord

import pytest

from gouge.colourcli import Simple


@pytest.mark.parametrize(
    "level,color_code",
    [
        (logging.DEBUG, "\x1b[30mDEBUG"),
        (logging.INFO, "\x1b[36mINFO"),
        (logging.WARNING, "\x1b[33mWARNING"),
        (logging.ERROR, "\x1b[31mERROR"),
        (logging.CRITICAL, "\x1b[41mCRITICAL"),
    ],
)
def test_debug(level, color_code):
    fmt = Simple(force_styling=True)
    record = LogRecord("name", level, "pathname", 1, "msg", (), None)
    result = fmt.format(record)
    assert color_code in result
    assert "msg" in result



import re
from gouge.colourcli import clr

P_UVICORN_ACCESS = re.compile(
    r'^(?P<remote_host>\S+) - "(?P<method>GET|POST|PUT|DELETE|PATCH|HEAD|OPTIONS) (?P<path>\S+) (?P<http>\S+)" (?P<status>\d+)'
)
METHOD_COLORS = {
    "GET": clr.Fore.GREEN,
    "POST": clr.Fore.YELLOW,
    "PUT": clr.Fore.YELLOW,
    "DELETE": clr.Fore.RED,
    "PATCH": clr.Fore.YELLOW,
    "HEAD": clr.Fore.GREEN,
    "OPTIONS": clr.Fore.GREEN,
}


def uvicorn_access(message: str) -> str:
    """
    A sample pre-formatter for the "uvicorn.access" logger
    """
    match = P_UVICORN_ACCESS.match(message)
    if match:
        remote_host, method, path, http_version, status_code = match.groups()
        if int(status_code) >= 500:
            status_color = f"{clr.Back.RED}{clr.Fore.YELLOW}"
        elif int(status_code) >= 400:
            status_color = clr.Fore.BLUE
        elif int(status_code) >= 300:
            status_color = clr.Fore.YELLOW
        else:
            status_color = clr.Fore.GREEN
        method_color = METHOD_COLORS.get(method, clr.Fore.YELLOW)
        return (
            f"{clr.Fore.BLUE}{remote_host}{clr.Style.RESET_ALL}"
            ' - "'
            f"{method_color}{method}{clr.Style.RESET_ALL} "
            f"{clr.Style.BRIGHT}{clr.Fore.WHITE}{path}{clr.Style.RESET_ALL} "
            f"{clr.Fore.CYAN}{http_version}{clr.Style.RESET_ALL}"
            '" '
            f"{status_color}{status_code}{clr.Style.RESET_ALL}"
        )
    return message



"""
This module tests the log formatter which colorises messages
"""

import logging
import re
from logging import LogRecord

from gouge.colourcli import Simple


def test_format_record_red_exception():
    """
    If the message level is higher or equal than "ERROR" we want to see the
    esception as red text. For lower levels we use another color.
    """
    record = LogRecord(
        "name",
        logging.ERROR,
        "path",
        42,
        "message",
        args={},
        exc_info=(ValueError, ValueError("foo"), None),
    )
    formatter = Simple(show_exc=True)
    needle = "\x1b[31mValueError"
    output = formatter.format(record)
    assert needle in output


def test_format_record_grey_exception():
    """
    If the message level is higher or equal than "ERROR" we want to see the
    esception as red text. For lower levels we use grey
    """
    record = LogRecord(
        "name",
        logging.DEBUG,
        "path",
        42,
        "message",
        args={},
        exc_info=(ValueError, ValueError("foo"), None),
    )
    formatter = Simple(show_exc=True)
    needle = "\x1b[37m\x1b[2mValueError"
    output = formatter.format(record)
    assert needle in output


def test_show_pid():
    """
    Ensure the PID is visible if requested
    """
    record = LogRecord(
        "name", logging.DEBUG, "path", 42, "message", args={}, exc_info=None
    )
    formatter = Simple(show_pid=True)
    output = formatter.format(record)
    assert re.search(r"\[PID: \d+\s*\]", output)


def test_dictconfig():
    """
    Ensure that we can configure loggers with dictConfig while using gouge
    """
    config = {
        "version": 1,
        "formatters": {
            "simple": {
                "class": "gouge.colourcli.Simple",
            },
        },
        "handlers": {
            "console": {
                "class": "logging.StreamHandler",
                "formatter": "simple",
            },
        },
        "loggers": {
            "": {
                "handlers": ["console"],
                "level": "DEBUG",
            },
        },
    }
    import logging.config

    logging.config.dictConfig(config)



import logging

from gouge.colourcli import Simple


def test_caplog(caplog):
    """
    Using the pytest "caplog" fixture can cause trouble with the "Simple"
    formatter. This test uses the fixture, and should not cause any errors.
    """
    Simple.basicConfig(level=logging.DEBUG)
    logging.warning("Hello World")
    assert len(caplog.records) == 1



import csv
import io
import logging
from typing import Any, Optional


class CSVLog(logging.Formatter):
    """
    CSV formatter for python loggers.

    The columns correspond to the following fields:

    * created
    * filename
    * funcName
    * levelname
    * levelno
    * lineno
    * module
    * msecs
    * name
    * pathname
    * process
    * processName
    * relativeCreated
    * thread
    * threadName
    * message
    * exc_text
    """

    @staticmethod
    def basicConfig(**kwargs: Any) -> None:
        logging.basicConfig(**kwargs)
        root = logging.getLogger()
        for handler in root.handlers:
            handler.setFormatter(CSVLog())

    def __init__(
        self, fmt: Optional[str] = None, datefmt: Optional[str] = None
    ) -> None:
        logging.Formatter.__init__(self, fmt, datefmt)
        self.buffer = io.StringIO()
        self.writer = csv.writer(self.buffer)

    def format(self, record: logging.LogRecord) -> str:
        record.message = record.getMessage()

        if record.exc_info:
            # Cache the traceback text to avoid converting it multiple times
            # (it's constant anyway)
            exc_text = getattr(record, "exc_text", "")
            if not exc_text:
                record.exc_text = self.formatException(record.exc_info)

        exc_text = getattr(record, "exc_text", "")
        message_items = [
            record.created,
            record.filename,
            record.funcName,
            record.levelname,
            record.levelno,
            record.lineno,
            record.module,
            record.msecs,
            record.name,
            record.pathname,
            record.process,
            record.processName,
            record.relativeCreated,
            record.thread,
            record.threadName,
            record.message,
            exc_text,
        ]

        self.writer.writerow(message_items)
        output = self.buffer.getvalue()
        self.buffer.seek(0)
        self.buffer.truncate()
        return output.strip()


class XMLLog(logging.Formatter):
    """
    XML formatter for python loggers.

    The records have the following schema::

        <record>
            <created />
            <filename />
            <funcName />
            <levelname />
            <levelno />
            <lineno />
            <module />
            <msecs />
            <name />
            <pathname />
            <process />
            <processName />
            <relativeCreated />
            <thread />
            <threadName />
            <message />
            <exc_text />
        </record>
    """

    @staticmethod
    def basicConfig(**kwargs: Any) -> None:
        logging.basicConfig(**kwargs)
        root = logging.getLogger()
        for handler in root.handlers:
            handler.setFormatter(XMLLog())

    def __init__(
        self, fmt: Optional[str] = None, datefmt: Optional[str] = None
    ) -> None:
        logging.Formatter.__init__(self, fmt, datefmt)
        from xml.dom.minidom import Document

        self.doc = Document()

    def format(self, record: logging.LogRecord) -> str:
        record.message = record.getMessage()

        if record.exc_info:
            # Cache the traceback text to avoid converting it multiple times
            # (it's constant anyway)
            exc_text = getattr(record, "exc_text", "")
            if not exc_text:
                record.exc_text = self.formatException(record.exc_info)

        exc_text = getattr(record, "exc_text", "")
        message_items = [
            ("created", record.created),
            ("filename", record.filename),
            ("funcName", record.funcName),
            ("levelname", record.levelname),
            ("levelno", record.levelno),
            ("lineno", record.lineno),
            ("module", record.module),
            ("msecs", record.msecs),
            ("name", record.name),
            ("pathname", record.pathname),
            ("process", record.process),
            ("processName", record.processName),
            ("relativeCreated", record.relativeCreated),
            ("thread", record.thread),
            ("threadName", record.threadName),
            ("message", record.message),
            ("exc_text", exc_text),
        ]

        element = self.doc.createElement("record")
        for tagname, value in message_items:
            subelement = self.doc.createElement(tagname)
            subelement.appendChild(self.doc.createTextNode(str(value)))
            element.appendChild(subelement)
        output: str = element.toxml()
        return output



from logging import LogRecord
import logging
from gouge.colourcli import Simple


def _dummy_preformatter(message: str) -> str:
    return "modified-message"


def test_preformatter():
    record = LogRecord(
        "the.logger",
        logging.INFO,
        "the-file",
        42,
        "the-message",
        None,
        None,
        None,
        None,
    )
    instance = Simple(pre_formatters={"the.logger": [_dummy_preformatter]})
    result = instance.format(record)
    assert "modified-message" in result



import io
import logging
from functools import partial

import pytest

from gouge.filters import ShiftingFilter

SimpleRecord = partial(
    logging.LogRecord, pathname="", lineno=0, msg="", args={}, exc_info=None
)


@pytest.fixture
def reset_logging():
    try:
        yield
    finally:
        logging.shutdown()
        for logger in logging.Logger.manager.loggerDict.values():
            if isinstance(logger, logging.PlaceHolder):
                continue
            del logger.handlers[:]
            del logger.filters[:]


def test_noshift():
    """
    A default instance should not shift anything
    """
    record = SimpleRecord(name="a.b.c", level=logging.DEBUG)
    filter_ = ShiftingFilter()
    filter_.filter(record)
    assert record.levelno == logging.DEBUG


def test_noshift_unmatching():
    """
    If the name does not match, no shifting should occur.
    """
    filter_ = ShiftingFilter(1, "foo.bar")
    record = SimpleRecord(name="a.b.c", level=logging.DEBUG)
    filter_.filter(record)
    assert record.levelno == logging.DEBUG


def test_noshift_parent():
    """
    Parent nodes should not be shifted.
    """
    filter_ = ShiftingFilter(1, "a.b.c")
    record = SimpleRecord(name="a.b", level=logging.DEBUG)
    filter_.filter(record)
    assert record.levelno == logging.DEBUG


def test_shift_fullmatch():
    """
    If the name matches exactly, we should shift
    """
    filter_ = ShiftingFilter(1, "a.b.c")
    record = SimpleRecord(name="a.b.c", level=logging.DEBUG)
    filter_.filter(record)
    assert record.levelno == logging.INFO


def test_shift_child():
    """
    Child nodes should be shifted
    """
    filter_ = ShiftingFilter(1, "a.b.c")
    record = SimpleRecord(name="a.b.c.d.e", level=logging.DEBUG)
    filter_.filter(record)
    assert record.levelno == logging.INFO


def test_shift_reversed():
    """
    We should also allow shifting in reverse.
    """
    filter_ = ShiftingFilter(-1)
    record = SimpleRecord(name="a.b.c", level=logging.INFO)
    filter_.filter(record)
    assert record.levelno == logging.DEBUG


def test_shift_fine_grained():
    """
    Instead of shifting by levels, the user should be allowed to shift by an
    exact offset.
    """
    filter_ = ShiftingFilter(offset=2)
    record = SimpleRecord(name="a.b.c", level=logging.NOTSET)
    filter_.filter(record)
    assert record.levelno == 2


def test_shift_min():
    """
    We should never go below level.NOTSET
    """
    filter_ = ShiftingFilter(-1)
    record = SimpleRecord(name="a.b.c", level=logging.NOTSET)
    filter_.filter(record)
    assert record.levelno == logging.NOTSET


def test_shift_max():
    """
    We also stop above logging.CRITICAL
    """
    filter_ = ShiftingFilter(1)
    record = SimpleRecord(name="a.b.c", level=logging.CRITICAL)
    filter_.filter(record)
    assert record.levelno == logging.CRITICAL


def test_shift_custom_min():
    """
    The user should have the option to refuse shifting below a certain
    level.
    """
    filter_ = ShiftingFilter(-1, min=logging.INFO)
    record = SimpleRecord(name="a.b.c", level=logging.INFO)
    filter_.filter(record)
    assert record.levelno == logging.INFO


def test_shift_custom_max():
    """
    The user should have the option to refuse shifting above a certain
    level.
    """
    filter_ = ShiftingFilter(1, max=logging.ERROR)
    record = SimpleRecord(name="a.b.c", level=logging.ERROR)
    filter_.filter(record)
    assert record.levelno == logging.ERROR


def test_shift_custom_min_outside_bounds():
    """
    We allow specifying a custom min value outside of the "standard" bounds.
    """
    filter_ = ShiftingFilter(-1, min=-10)
    record = SimpleRecord(name="a.b.c", level=logging.NOTSET)
    filter_.filter(record)
    assert record.levelno == -10


def test_shift_custom_max_outside_bounds():
    """
    We allow specifying a custom max value outside of the "standard" bounds.
    """
    filter_ = ShiftingFilter(1, max=60)
    record = SimpleRecord(name="a.b.c", level=logging.CRITICAL)
    filter_.filter(record)
    assert record.levelno == 60


def test_attached_to_handler(reset_logging):
    blob = io.StringIO()
    handler = logging.StreamHandler(blob)
    handler.setFormatter(logging.Formatter("%(levelno)s %(levelname)s %(msg)s"))
    handler.addFilter(ShiftingFilter(-1))
    logger_c = logging.getLogger("a.b.c")
    logger_d = logging.getLogger("a.b.d")
    logger_parent = logging.getLogger("a")
    logger_parent.setLevel(logging.DEBUG)
    logger_parent.addHandler(handler)

    logger_c.error("error - c")
    logger_d.info("info - d")
    logger_d.debug("debug - d")

    lines = blob.getvalue().splitlines()

    expected = [
        "30 WARNING error - c",
        "10 DEBUG info - d",
        "0 NOTSET debug - d",
    ]
    assert lines == expected


def test_attached_to_logger(reset_logging):
    blob = io.StringIO()
    handler = logging.StreamHandler(blob)
    handler.setFormatter(logging.Formatter("%(levelno)s %(levelname)s %(msg)s"))

    logger_c = logging.getLogger("a.b.c")
    logger_d = logging.getLogger("a.b.d")
    logger_parent = logging.getLogger("a")
    logger_parent.setLevel(logging.DEBUG)
    logger_parent.addHandler(handler)

    filter = ShiftingFilter(-1)
    filter.inject("a")

    logger_c.error("error - c")
    logger_d.info("info - d")
    logger_d.debug("debug - d")

    filter.cleanup()

    lines = blob.getvalue().splitlines()

    expected = [
        "30 WARNING error - c",
        "10 DEBUG info - d",
        "0 NOTSET debug - d",
    ]
    assert lines == expected
